<?php

/*
 * This file is part of a XenForo add-on.
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */

namespace SV\ModeratorEssentials\XF\ModeratorLog;

use SV\ModeratorEssentials\ModeratorLog\PanelInterface;

use XF\Entity\ModeratorLog;

/**
 * Extends \XF\ModeratorLog\Post
 */
class Post extends XFCP_Post implements PanelInterface
{
    /**
     * @return bool
     */
    public function canViewLogs()
    {
        return true;
    }

    /**
     * @param ModeratorLog $log
     * @param string       $error
     * @return bool
     */
    public function canViewLog(ModeratorLog $log, &$error = null)
    {
        /** @var \SV\ModeratorEssentials\XF\Entity\ModeratorLog $log */
        $action = $log->action;
        if (!in_array($action, $this->getActions()))
        {
            return false;
        }

        /** @var \XF\Entity\Post $post */
        $post = $log->Content;
        if (!$post)
        {
            return false;
        }

        $thread = $post->Thread;
        if (!$thread)
        {
            return false;
        }

        /** @var \SV\ModeratorEssentials\XF\Entity\Forum $forum */
        $forum = $thread->Forum;
        if (!$forum)
        {
            return false;
        }

        return $forum->canViewModeratorLogEntries();
    }

    /**
     * @return bool
     */
    public function requiresHydration()
    {
        return true;
    }

    /**
     * @return array
     */
    public function getActions()
    {
        return [
            'approve',
            'convert_to_conversation',
            'delete_hard',
            'delete_soft',
            'edit',
            'merge_target',
            'unapprove',
            'undelete',
            'warning_expired',
            'warning_given',
            'warning_removed'
        ];
    }

    /**
     * @return array
     */
    public function getEntityWith()
    {
        $with = parent::getEntityWith();

        $with[] = 'Thread';
        $with[] = 'Thread.Forum';
        $visitor = \XF::visitor();
        $with[] = 'Thread.Forum.Node.Permissions|' . $visitor->permission_combination_id;

        return $with;
    }
}
